'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('../_virtual/_rollupPluginBabelHelpers.cjs');
var PublicKey = require('../PublicKey.cjs');
var SdkError = require('./SdkError.cjs');

/** @group Errors */
class InterfaceImplementationMissingError extends SdkError.SdkError {
  constructor(interfaceName, contextVariable) {
    const interfaceBasename = interfaceName.replace(/Interface$/, '');
    const message = `Tried using ${interfaceName} but no implementation of that interface was found. ` + `Make sure an implementation is registered, ` + `e.g. via "context.${contextVariable} = new My${interfaceBasename}();".`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'InterfaceImplementationMissingError');
  }
}

/** @group Errors */
class UnexpectedAmountError extends SdkError.SdkError {
  constructor(amount, expectedIdentifier, expectedDecimals) {
    const message = `Expected amount of type [${expectedIdentifier} with ${expectedDecimals} decimals] ` + `but got [${amount.identifier} with ${amount.decimals} decimals]. ` + `Ensure the provided Amount is of the expected type.`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'UnexpectedAmountError');
    this.amount = amount;
    this.expectedIdentifier = expectedIdentifier;
    this.expectedDecimals = expectedDecimals;
  }
}

/** @group Errors */
class AmountMismatchError extends SdkError.SdkError {
  constructor(left, right, operation) {
    const wrappedOperation = operation ? ` [${operation}]` : '';
    const message = `The SDK tried to execute an operation${wrappedOperation} on two amounts of different types: ` + `[${left.identifier} with ${left.decimals} decimals] and ` + `[${right.identifier} with ${right.decimals} decimals]. ` + `Provide both amounts in the same type to perform this operation.`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'AmountMismatchError');
    this.left = left;
    this.right = right;
    this.operation = operation;
  }
}

/** @group Errors */
class InvalidJsonVariableError extends SdkError.SdkError {
  constructor(cause) {
    super('The provided JSON variable could not be parsed into a string.', cause);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'InvalidJsonVariableError');
  }
}

/** @group Errors */
class InvalidJsonStringError extends SdkError.SdkError {
  constructor(cause) {
    super('The provided string could not be parsed into a JSON variable.', cause);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'InvalidJsonStringError');
  }
}

/** @group Errors */
class OperationUnauthorizedForGuestsError extends SdkError.SdkError {
  constructor(operation) {
    const message = `Trying to access the [${operation}] operation as a guest. ` + `Ensure your wallet is connected using the identity driver. ` + `For instance, by using "metaplex.use(walletAdapterIdentity(wallet))" or ` + `"metaplex.use(keypairIdentity(keypair))".`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'OperationUnauthorizedForGuestsError');
  }
}

/** @group Errors */
class AssetNotFoundError extends SdkError.SdkError {
  constructor(location) {
    super(`The asset at [${location}] could not be found.`);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'AssetNotFoundError');
  }
}

/** @group Errors */
class AccountNotFoundError extends SdkError.SdkError {
  constructor(address, accountType, solution) {
    const message = `${accountType ? `The account of type [${accountType}] was not found` : 'No account was found'} at the provided address [${PublicKey.formatPublicKey(address)}].${solution ? ` ${solution}` : ''}`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'AccountNotFoundError');
  }
}

/** @group Errors */
class UnexpectedAccountError extends SdkError.SdkError {
  constructor(address, expectedType, cause) {
    const message = `The account at the provided address [${PublicKey.formatPublicKey(address)}] ` + `is not of the expected type [${expectedType}].`;
    super(message, cause);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'UnexpectedAccountError');
  }
}

/** @group Errors */
class NotYetImplementedError extends SdkError.SdkError {
  constructor() {
    const message = `This feature is not yet implemented. Please check back later.`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'NotYetImplementedError');
  }
}

/** @group Errors */
class UnreachableCaseError extends SdkError.SdkError {
  constructor(value) {
    const message = `A switch statement is not handling the provided case [${value}]. ` + `Check your inputs or raise an issue to have ensure all cases are handled properly.`;
    super(message);
    _rollupPluginBabelHelpers.defineProperty(this, "name", 'UnreachableCaseError');
  }
}

exports.AccountNotFoundError = AccountNotFoundError;
exports.AmountMismatchError = AmountMismatchError;
exports.AssetNotFoundError = AssetNotFoundError;
exports.InterfaceImplementationMissingError = InterfaceImplementationMissingError;
exports.InvalidJsonStringError = InvalidJsonStringError;
exports.InvalidJsonVariableError = InvalidJsonVariableError;
exports.NotYetImplementedError = NotYetImplementedError;
exports.OperationUnauthorizedForGuestsError = OperationUnauthorizedForGuestsError;
exports.UnexpectedAccountError = UnexpectedAccountError;
exports.UnexpectedAmountError = UnexpectedAmountError;
exports.UnreachableCaseError = UnreachableCaseError;
//# sourceMappingURL=errors.cjs.map
