import { deduplicateSigners, signTransaction } from './Signer.mjs';

class TransactionBuilder {
  constructor(context, items = []) {
    this.context = context;
    this.items = items;
  }
  prepend(input) {
    return new TransactionBuilder(this.context, [...this.parseItems(input), ...this.items]);
  }
  append(input) {
    return new TransactionBuilder(this.context, [...this.items, ...this.parseItems(input)]);
  }
  add(input) {
    return this.append(input);
  }
  splitAtIndex(index) {
    return [new TransactionBuilder(this.context, this.items.slice(0, index)), new TransactionBuilder(this.context, this.items.slice(index))];
  }
  getInstructions() {
    return this.items.map(item => item.instruction);
  }
  getSigners() {
    return deduplicateSigners([this.context.payer, ...this.items.flatMap(item => item.signers)]);
  }
  getBytesCreatedOnChain() {
    return this.items.reduce((sum, item) => sum + item.bytesCreatedOnChain, 0);
  }
  async getRentCreatedOnChain() {
    return this.context.rpc.getRent(this.getBytesCreatedOnChain(), {
      includesHeaderBytes: true
    });
  }
  build(options) {
    return this.context.transactions.create({
      payer: this.context.payer.publicKey,
      instructions: this.getInstructions(),
      ...options
    });
  }
  async buildAndSign(options) {
    return signTransaction(this.build(options), this.getSigners());
  }
  async send(options = {}) {
    const blockhash = options.build?.blockhash ?? (await this.context.rpc.getLatestBlockhash()).blockhash;
    const transaction = await this.buildAndSign({
      blockhash,
      ...options.build
    });
    return this.context.rpc.sendTransaction(transaction, options.send);
  }
  async sendAndConfirm(options = {}) {
    let blockhash;
    let strategy;
    if (options.confirm?.strategy && options.build?.blockhash) {
      blockhash = options.build.blockhash;
      strategy = options.confirm.strategy;
    } else {
      const latestBlockhash = await this.context.rpc.getLatestBlockhash();
      blockhash = latestBlockhash.blockhash;
      strategy = options.confirm?.strategy ?? {
        type: 'blockhash',
        ...latestBlockhash
      };
    }
    const signature = await this.send({
      ...options,
      build: {
        blockhash,
        ...options.build
      }
    });
    const result = await this.context.rpc.confirmTransaction(signature, {
      strategy,
      ...options.confirm
    });
    return {
      signature,
      result
    };
  }
  parseItems(input) {
    return (Array.isArray(input) ? input : [input]).flatMap(item => 'instruction' in item ? [item] : item.items);
  }
}
const transactionBuilder = (context, items = []) => new TransactionBuilder(context, items);

export { TransactionBuilder, transactionBuilder };
//# sourceMappingURL=TransactionBuilder.mjs.map
